@@ -10,9 +10,8 @@ import numpy as np
 from graph_tool import all as gt
 from graph_tool.topology import label_components
 from collections import Counter
+import random
 
-
-# TODO: the random_start walk if arg set to true
 def random_walks(
     attractor_dict,
     rules,
@@ -25,7 +24,7 @@ def random_walks(
     max_steps=500,
     stability=False,
     reach_or_leave="leave",
-    random_start=False,
+    random_start=0,
     on_nodes=None,
     off_nodes=None,
     basin=0,
@@ -57,8 +56,8 @@ def random_walks(
         Whether to perform stability testing with multiple radii or not
     reach_or_leave : string
         Define what type of random walk to perform
-    random_start: bool
-        Whether to perform walks with a random list of start states
+    random_start: int
+        Whether to perform walks with a random list of start states. If >0, run that many random starts
     on_nodes : list
         Define ON nodes of a perturbation
     off_nodes : list
@@ -129,11 +128,11 @@ def random_walks(
                 # Print progress of random walk every 10% of the way through iters
                 prog = 0
                 for iter_ in range(iters):
-                    print("Iteration:", iter_)
-                    print("Progress:")
-                    if iter_ / 10 > prog:
+                    # print("Iteration:", iter_)
+                    # print("Progress:")
+                    if iter_ % 100 == 0:
                         prog = iter_ / 10
-                        print(prog)
+                        print("Progress: ", prog)
 
                     # 'counts': histogram of walk
                     # 'switches': count which TFs flipped
@@ -185,116 +184,209 @@ def random_walks(
                 outfile.close()
                 out_len.close()
 
-            # Perform walks with perturbations
-            if perturbations:
-                try:
-                    os.mkdir(op.join(save_dir, "perturbations"))
-                except FileExistsError:
-                    pass
-                # Run all possible single perturbations
-                if on_nodes == None and off_nodes == None:
+                # Perform walks with perturbations
+                if perturbations:
                     try:
-                        os.mkdir(op.join(save_dir, "perturbations/%d" % start_idx))
+                        os.mkdir(op.join(save_dir, "perturbations"))
                     except FileExistsError:
                         pass
+                    # Run all possible single perturbations
+                    if on_nodes == None and off_nodes == None:
+                        try:
+                            os.mkdir(op.join(save_dir, "perturbations/%d" % start_idx))
+                        except FileExistsError:
+                            pass
 
-                    outfile = open(
-                        op.join(save_dir, f"perturbations/%d/results.csv" % start_idx),
-                        "w+",
-                    )
+                        outfile = open(
+                            op.join(save_dir, f"perturbations/%d/results.csv" % start_idx),
+                            "w+",
+                        )
 
-                    for expt_node in nodes:
-                        # Arrays of # steps when activating or knocking out
-                        n_steps_activate = []
-                        n_steps_knockout = []
-                        prog = 0
+                        for expt_node in nodes:
+                            # Arrays of # steps when activating or knocking out
+                            n_steps_activate = []
+                            n_steps_knockout = []
+                            prog = 0
 
-                        expt = "%s_activate" % expt_node
-                        for iter_ in range(iters):
-                            if iter_ / 10 > prog:
-                                prog = iter_ / 10
-                                print(prog)
-                            # To perturb more than one node, add to on_nodes or off_nodes
-                            if reach_or_leave == "leave":
+                            expt = "%s_activate" % expt_node
+                            for iter_ in range(iters):
+                                if iter_ % 100 == 0:
+                                    prog = iter_ / 10
+                                    print("Progress: ", prog)
+                                # To perturb more than one node, add to on_nodes or off_nodes
+                                if reach_or_leave == "leave":
+                                    (
+                                        walk_on,
+                                        counts_on,
+                                        switches_on,
+                                        distances_on,
+                                    ) = random_walk_until_leave_basin(
+                                        start_idx,
+                                        rules,
+                                        regulators_dict,
+                                        nodes,
+                                        radius,
+                                        max_steps=max_steps,
+                                        on_nodes=[
+                                            expt_node,
+                                        ],
+                                        off_nodes=[],
+                                    )
+                                # elif reach_or_leave == "reach":
+
+                                n_steps_activate.append(len(distances_on))
+
+                            # mean of non-perturbed vs perturbed: loc_0 and loc_1
+                            # histogram plots: inverse gaussian?
+                            loc_0, loc_1, stabilized = plot_histograms(
+                                n_steps_to_leave_0,
+                                n_steps_activate,
+                                expt,
+                                bins=60,
+                                fname=op.join(
+                                    save_dir, "perturbations/%d/%s.pdf" % (start_idx, expt)
+                                ),
+                            )
+
+                            outfile.write(
+                                op.join(
+                                    save_dir,
+                                    "perturbations/%d,%s,%s,activate,%f\n"
+                                    % (start_idx, k, expt_node, stabilized),
+                                )
+                            )
+                            expt = "%s_knockdown" % expt_node
+                            for iter_ in range(iters):
                                 (
-                                    walk_on,
-                                    counts_on,
-                                    switches_on,
-                                    distances_on,
+                                    walk_off,
+                                    counts_off,
+                                    switches_off,
+                                    distances_off,
                                 ) = random_walk_until_leave_basin(
                                     start_idx,
                                     rules,
                                     regulators_dict,
                                     nodes,
                                     radius,
-                                    max_steps=500,
-                                    on_nodes=[
+                                    max_steps=max_steps,
+                                    on_nodes=[],
+                                    off_nodes=[
                                         expt_node,
                                     ],
-                                    off_nodes=[],
                                 )
-                            # elif reach_or_leave == "reach":
 
-                            n_steps_activate.append(len(distances_on))
+                                n_steps_knockout.append(len(distances_off))
 
-                        # mean of non-perturbed vs perturbed: loc_0 and loc_1
-                        # histogram plots: inverse gaussian?
-                        loc_0, loc_1, stabilized = plot_histograms(
-                            n_steps_to_leave_0,
-                            n_steps_activate,
-                            expt,
-                            bins=60,
-                            fname=op.join(
-                                save_dir, "perturbations/%d/%s.pdf" % (start_idx, expt)
-                            ),
-                        )
-
-                        outfile.write(
-                            op.join(
-                                save_dir,
-                                "perturbations/%d,%s,%s,activate,%f\n"
-                                % (start_idx, k, expt_node, stabilized),
+                            loc_0, loc_1, stabilized = plot_histograms(
+                                n_steps_to_leave_0,
+                                n_steps_knockout,
+                                expt,
+                                bins=60,
+                                fname=op.join(
+                                    save_dir, "perturbations/%d/%s.pdf" % (start_idx, expt)
+                                ),
                             )
-                        )
-                        expt = "%s_knockdown" % expt_node
-                        for iter_ in range(iters):
-                            (
-                                walk_off,
-                                counts_off,
-                                switches_off,
-                                distances_off,
-                            ) = random_walk_until_leave_basin(
-                                start_idx,
-                                rules,
-                                regulators_dict,
-                                nodes,
-                                radius,
-                                max_steps=500,
-                                on_nodes=[],
-                                off_nodes=[
-                                    expt_node,
-                                ],
+                            outfile.write(
+                                op.join(
+                                    save_dir,
+                                    "perturbations/%d,%s,%s,knockdown,%f\n"
+                                    % (start_idx, k, expt_node, stabilized),
+                                )
                             )
+                        outfile.close()
+    
+    if random_start > 0:
+        try:
+            os.mkdir(op.join(save_dir, '/walks/random'))
+        except FileExistsError:
+            pass
+        random_list = []
+        for i in range(random_start):
+            rand_state = random.choices([0,1], k=len(nodes))
+            rand_idx = ut.state_bool2idx(rand_state)
+            random_list.append(rand_idx)
+        for radius_ in radius:
+            for start_idx in random_list:
+                switch_counts_0 = dict()
+                for node in nodes:
+                    switch_counts_0[node] = 0
+                n_steps_to_leave_0 = []
+                try:
+                    os.mkdir(op.join(save_dir, "walks/random/%d" % start_idx))
+                except FileExistsError:
+                    pass
 
-                            n_steps_knockout.append(len(distances_off))
+                outfile = open(
+                    op.join(
+                        save_dir, f"walks/random/%d/results_radius_{radius_}.csv" % start_idx
+                    ),
+                    "w+",
+                )
+                out_len = open(
+                    op.join(save_dir, f"walks/random/%d/len_walks_{radius_}.csv" % start_idx),
+                    "w+",
+                )
 
-                        loc_0, loc_1, stabilized = plot_histograms(
-                            n_steps_to_leave_0,
-                            n_steps_knockout,
-                            expt,
-                            bins=60,
-                            fname=op.join(
-                                save_dir, "perturbations/%d/%s.pdf" % (start_idx, expt)
-                            ),
+                # Perform walks without perturbations first
+                # Print progress of random walk every 10% of the way through iters
+                prog = 0
+                for iter_ in range(iters):
+                    # print("Iteration:", iter_)
+                    # print("Progress:")
+                    if iter_ % 100 == 0:
+                        prog = iter_ / 10
+                        print("Progress: ", prog)
+
+                    # 'counts': histogram of walk
+                    # 'switches': count which TFs flipped
+                    # 'distance': starting state to current state; walk until take max steps or leave basin
+                    if reach_or_leave == "leave":
+                        (
+                            walk,
+                            counts,
+                            switches,
+                            distances,
+                        ) = random_walk_until_leave_basin(
+                            start_idx,
+                            rules,
+                            regulators_dict,
+                            nodes,
+                            radius_,
+                            max_steps=max_steps,
+                            on_nodes=on_nodes,
+                            off_nodes=off_nodes,
                         )
-                        outfile.write(
-                            op.join(
-                                save_dir,
-                                "perturbations/%d,%s,%s,knockdown,%f\n"
-                                % (start_idx, k, expt_node, stabilized),
-                            )
+                    elif reach_or_leave == "reach":
+                        (
+                            walk,
+                            counts,
+                            switches,
+                            distances,
+                        ) = random_walk_until_reach_basin(
+                            start_idx,
+                            rules,
+                            regulators_dict,
+                            nodes,
+                            radius=radius_,
+                            max_steps=max_steps,
+                            on_nodes=on_nodes,
+                            off_nodes=off_nodes,
+                            basin=basin,
                         )
-                    outfile.close()
+                    else:
+                        raise Exception(
+                            "Value for `reach_or_leave` must be string 'reach' or 'leave'."
+                        )
+
+                    n_steps_to_leave_0.append(len(distances))
+                    for node in switches:
+                        if node is not None:
+                            switch_counts_0[node] += 1
+                    outfile.write(f"{walk}\n")
+                    out_len.write(f"{len(walk)}\n")
+                outfile.close()
+                out_len.close()
+
 
 
 def simple_random_walk(stg, edge_weights, start_idx, steps):
